package com.sksamuel.scoot.rest.client

import com.sun.jersey.api.client.{GenericType, ClientResponse}
import scala.collection.JavaConverters._
import com.sksamuel.scoot.rest.Logging
import org.apache.commons.io.IOUtils

/** @author Stephen Samuel */
class Response(resp: ClientResponse) extends Logging {

    def header(name: String): Option[String] = Option(resp.getHeaders.getFirst(name))
    def cookie(name: String): Option[Cookie] = resp.getCookies.asScala.find(_.getName == name)
      .map(arg => Cookie(arg.getName, arg.getValue, arg.getPath, arg.getMaxAge, arg.getDomain))
    def length = header("Content-Length")
    def contentType = header("Content-Type")
    def status: Int = resp.getStatus

    def bytes = _safeInvoke
    def unmarshall[T: Manifest] = _safeInvoke(Left(manifest.erasure.asInstanceOf[Class[T]]))
    def unmarshall[T](c: Class[T]): Option[T] = _safeInvoke(Left(c))
    def unmarshall[T](gt: GenericType[T]): Option[T] = _safeInvoke(Right(gt))

    def close() {
        try {
            resp.close()
        } catch {
            case e: Exception => {
                logger.warn("Error closing client", e)
            }
        }
    }

    // return the contents of the request as a byte array
    private def _safeInvoke: Array[Byte] = {
        try {
            resp.getStatus match {
                case 200 => IOUtils.toByteArray(resp.getEntityInputStream)
                case _ => Array()
            }
        } catch {
            case e: Exception => {
                logger.warn("Error marshalling content", e)
                Array()
            }
        } finally {
            close()
        }
    }

    private def _safeInvoke[T](either: Either[Class[T], GenericType[T]]): Option[T] = {
        try {
            resp.getStatus match {
                case 200 => _content(either)
                case i: Int => None
            }
        } catch {
            case e: Exception => {
                logger.warn("Error marshalling content", e)
                None
            }
        } finally {
            close()
        }
    }

    private def _content[T](either: Either[Class[T], GenericType[T]]): Option[T] = {
        val entity = either match {
            case Left(x) => {
                logger.debug("status 200 marshalling into {}", x)
                resp.getEntity(x)
            }
            case Right(x) => {
                logger.debug("status 200 marshalling into {}", x)
                resp.getEntity(x)
            }
        }
        Option(entity)
    }
}

case class Cookie(name: String, value: String, path: String, maxAge: Int, domain: String)